@@ -1,4 +1,4 @@
-(defproject puppetlabs/trapperkeeper-metrics "1.1.0"
+(defproject puppetlabs/trapperkeeper-metrics "1.2.1"
   :description "Trapperkeeper Metrics Service"
   :url "http://github.com/puppetlabs/trapperkeeper-metrics"
 
@@ -6,7 +6,7 @@
 
   :pedantic? :abort
 
-  :parent-project {:coords [puppetlabs/clj-parent "1.2.0"]
+  :parent-project {:coords [puppetlabs/clj-parent "1.7.26"]
                    :inherit [:managed-dependencies]}
 
   :dependencies [[org.clojure/clojure]
@@ -20,18 +20,12 @@
                  [cheshire]
                  [org.clojure/java.jmx]
 
-                 ;; ring-defaults brings in a bad, old version of the servlet-api, which
-                 ;; now has a new artifact name (javax.servlet/javax.servlet-api).  If we
-                 ;; don't exclude the old one here, they'll both be brought in, and consumers
-                 ;; will be subject to the whims of which one shows up on the classpath first.
-                 ;; thus, we need to use exclusions here, even though we'd normally resolve
-                 ;; this type of thing by just specifying a fixed dependency version.
-                 [ring/ring-defaults nil :exclusions [javax.servlet/servlet-api]]
+                 [ring/ring-defaults]
 
                  [org.clojure/tools.logging]
                  [io.dropwizard.metrics/metrics-core]
                  [io.dropwizard.metrics/metrics-graphite]
-                 [org.jolokia/jolokia-core "1.3.5"]
+                 [org.jolokia/jolokia-core "1.6.2"]
                  [puppetlabs/comidi]
                  [puppetlabs/i18n]]
 
@@ -41,9 +35,9 @@
   :source-paths  ["src/clj"]
   :java-source-paths  ["src/java"]
 
-  :deploy-repositories [["releases" {:url "https://clojars.org/repo"
-                                     :username :env/clojars_jenkins_username
-                                     :password :env/clojars_jenkins_password
+  :deploy-repositories [["releases" {:url "https://artifactory.delivery.puppetlabs.net/artifactory/clojure-releases__local/"
+                                     :username :env/nexus_jenkins_username
+                                     :password :env/nexus_jenkins_password
                                      :sign-releases false}]]
 
   :classifiers  [["test" :testutils]]
