@@ -72,7 +72,7 @@
             [puppetlabs.trapperkeeper.services :refer [service-id service-context]]
             [robert.hooke :as rh]
             [schema.core :as s]
-            [slingshot.slingshot :refer [throw+ try+]]
+            [slingshot.slingshot :refer [throw+]]
             [clojure.core.async :as async]
             [puppetlabs.puppetdb.command :as cmd]
             [puppetlabs.puppetdb.queue :as queue]
@@ -278,6 +278,12 @@
         (finally
           (.unlock lock))))))
 
+(defn- delete-node-from-puppetdb [context certname]
+  "Implements the PuppetDBServer delete-node method, see the protocol
+   for further information"
+  (jdbc/with-transacted-connection (get-in context [:shared-globals :scf-write-db])
+    (scf-store/delete-certname! certname)))
+
 (defn maybe-check-for-updates
   [config read-db job-pool]
   (if (conf/foss? config)
@@ -312,10 +318,45 @@
     (future-cancel command-loader))
   context)
 
+;; A map of required postgres settings and their expected value.
+;; PuppetDB will refuse to start unless ALL of these values
+;; match the actual database configuration settings.
+(def required-pg-settings
+  {:standard_conforming_strings "on"})
+
+(defn verify-database-setting
+  "Given m, a map of database settings, verify 'expected', which has the form
+  [setting-name expected-value]."
+  [m [setting value]]
+  (let [map-path [setting :setting]]
+    (when-not (= (get-in m map-path) value)
+      {setting {:expected value :actual (get-in m map-path)}})))
+
+(defn request-database-settings
+  [db-conn-pool]
+  (jdbc/with-db-connection
+    db-conn-pool
+    (jdbc/query "show all;" )))
+
+(defn verify-database-settings
+  "Ensure the database configuration does not have any settings known
+  to break PuppetDB. If an invalid database configuration is found, throws
+  {:kind ::invalid-database-configuration :failed-validation failed-map}"
+  [database-settings]
+  (let [munged-settings (apply merge
+                               (map (fn [h]
+                                      {(keyword (:name h)) (dissoc h :name)})
+                                    database-settings))
+        invalid-settings (remove nil? (map #(verify-database-setting munged-settings %) required-pg-settings))]
+    (when (seq invalid-settings)
+      (throw (ex-info "One or more postgres settings failed validation"
+                      {:kind ::invalid-database-configuration
+                       :failed-validation (apply merge invalid-settings)})))))
+
 (defn initialize-schema
   "Ensures the database is migrated to the latest version, and returns
   true iff any migrations were run.  Throws
-  {:type ::unsupported-database :current version :oldest version} if
+  {:kind ::unsupported-database :current version :oldest version} if
   the current database is not supported."
   [db-conn-pool config]
   (jdbc/with-db-connection db-conn-pool
@@ -323,9 +364,10 @@
           oldest (get-in config [:database :min-required-version]
                            scf-store/oldest-supported-db)]
       (when (neg? (compare current oldest))
-        (throw+ {:type ::unsupported-database
-                 :current current
-                 :oldest oldest}))
+        (throw (ex-info "Database version too old"
+                        {:kind ::unsupported-database
+                         :current current
+                         :oldest oldest})))
       @sutils/db-metadata
       (let [migrated? (migrate! db-conn-pool)]
         (indexes! config)
@@ -336,8 +378,10 @@
   connection using a transient connection pool derived from the
   `write-db-config`.  Blocks until the initialization is complete, and
   then returns an unspecified value.  Throws exceptions on errors.
-  Throws {:type ::unsupported-database :current version :oldest
-  version} if the current database is not supported."
+  Throws {:kind ::unsupported-database :current version :oldest
+  version} if the current database is not supported. Throws
+  {:kind ::invalid-database-configuration :failed-validation failed-map}
+  if the database contains a disallowed setting."
   [write-db-config config]
   ;; A C-c (SIGINT) will close the pool via the shutdown hook, which
   ;; will then cause the pool connection to throw a (generic)
@@ -359,6 +403,7 @@
         (loop [i 0
                last-ex nil]
           (let [result (try
+                         (verify-database-settings (request-database-settings {:datasource db-pool}))
                          (initialize-schema {:datasource db-pool} config)
                          true
                          (catch java.sql.SQLTransientConnectionException ex
@@ -429,8 +474,9 @@
     (async/>!! cmd-event-ch (queue/make-cmd-event cmdref kind))))
 
 (defn start-puppetdb
-  "Throws {:type ::unsupported-database :current version :oldest version} if
-  the current database is not supported."
+  "Throws {:kind ::unsupported-database :current version :oldest version} if
+  the current database is not supported. If a database setting is configured
+  incorrectly, throws {:kind ::invalid-database-configuration :failed-validation failed-map}"
   [context config service get-registered-endpoints]
   {:pre [(map? context)
          (map? config)]
@@ -440,7 +486,7 @@
                 puppetdb command-processing
                 emit-cmd-events?]} config
         {:keys [pretty-print max-enqueued]} developer
-        {:keys [gc-interval]} database
+        {:keys [gc-interval node-purge-ttl]} database
         {:keys [disable-update-checking]} puppetdb
         {:keys [cmd-event-mult cmd-event-ch]} context
 
@@ -462,7 +508,8 @@
           command-chan (async/chan
                          (queue/sorted-command-buffer
                           max-enqueued
-                          #(cmd/update-counter! :invalidated %1 %2 inc!)))
+                          (fn [cmd ver] (cmd/update-counter! :invalidated cmd ver inc!))
+                          (fn [cmd ver] (cmd/update-counter! :ignored cmd ver inc!))))
           emit-cmd-events? (or (conf/pe? config) emit-cmd-events?)
           maybe-send-cmd-event! (partial maybe-send-cmd-event! emit-cmd-events? cmd-event-ch)
           [q load-messages] (queue/create-or-open-stockpile (conf/stockpile-dir config)
@@ -475,6 +522,7 @@
                    :dlo (dlo/initialize (get-path stockdir "discard")
                                         (get-in metrics-registries
                                                 [:dlo :registry]))
+                   :node-purge-ttl node-purge-ttl
                    :command-chan command-chan
                    :cmd-event-mult cmd-event-mult
                    :maybe-send-cmd-event! maybe-send-cmd-event!}
@@ -514,6 +562,31 @@
        (first oldest)
        (second oldest)))
 
+(defn invalid-conf-msg
+  [invalid-settings]
+  (str (trs "Invalid database configuration settings: ")
+       (str/join ", "
+                  ;; This is used to construct a list of invalid database settings
+                  ;;
+                  ;; 0 : The name of a database setting
+                  ;; 1 : The expected value of that database setting
+                  ;; 2 : The actual value of that database setting
+                  (map (fn [[k v]] (trs "''{0}'' (expected ''{1}'', got ''{2}'')" (name k) (:expected v) (:actual v)))
+                       invalid-settings))))
+
+(defn log-start-error
+  [e]
+  (let [data (ex-data e)
+        kind (:kind data)
+        msg (case kind
+              ::unsupported-database (db-unsupported-msg (:current data) (:oldest data))
+              ::invalid-database-configuration (invalid-conf-msg (:failed-validation data))
+              (trs "Unkown fatal start error {0}" e))
+        attn (utils/attention-msg msg)]
+    (utils/println-err attn)
+    (log/error attn)
+    msg))
+
 (defprotocol PuppetDBServer
   (shared-globals [this])
   (set-url-prefix [this url-prefix])
@@ -537,7 +610,9 @@
      and :producer-ts entries for each command PDB has either ingested or
      finished processing. The {:kind ::queue/queue-loaded} map is used to
      indicate that the queue/message-loader has finished loading any existing
-     commands from the stockpile queue when PDB starts up."))
+     commands from the stockpile queue when PDB starts up.")
+  (delete-node [this certname]
+    "Immediately delete all data for the provided certname"))
 
 (defservice puppetdb-service
   "Defines a trapperkeeper service for PuppetDB; this service is responsible
@@ -545,8 +620,7 @@
   that trapperkeeper will call on exit."
   PuppetDBServer
   [[:DefaultedConfig get-config]
-   [:WebroutingService add-ring-handler get-registered-endpoints]
-   [:ShutdownService request-shutdown shutdown-on-error]]
+   [:WebroutingService add-ring-handler get-registered-endpoints]]
   (init [this context]
 
         (doseq [{:keys [reporter]} (vals metrics/metrics-registries)]
@@ -561,18 +635,13 @@
                  ;; support jdk < 10.
                  ;; https://bugs.openjdk.java.net/browse/JDK-8176254
                  :stop-status (atom #{}))))
-  (start [this context]
-         (try+
-          (start-puppetdb context (get-config) this get-registered-endpoints)
-          (catch [:type ::unsupported-database] {:keys [current oldest]}
-            (let [msg (db-unsupported-msg current oldest)
-                  attn (utils/attention-msg msg)]
-              (utils/println-err attn)
-              (log/error attn)
-              ;; Until TK-445 is resolved, we won't be able to avoid the
-              ;; backtrace on exit this causes.
-              (shutdown-on-error (service-id this)
-                                 #(throw (Exception. msg)))))))
+  (start
+   [this context]
+   (try
+     (start-puppetdb context (get-config) this get-registered-endpoints)
+     (catch clojure.lang.ExceptionInfo e
+       (let [msg (log-start-error e)]
+         (throw (Exception. msg))))))
 
   (stop [this context]
         (doseq [{:keys [reporter]} (vals metrics/metrics-registries)]
@@ -596,7 +665,7 @@
   (query [this version query-expr paging-options row-callback-fn]
          (let [sc (service-context this)
                query-options (-> (get sc :shared-globals)
-                                 (select-keys [:scf-read-db :warn-experimental])
+                                 (select-keys [:scf-read-db :warn-experimental :node-purge-ttl])
                                  (assoc :url-prefix @(get sc :url-prefix)))]
            (qeng/stream-query-result version
                                      query-expr
@@ -606,6 +675,9 @@
   (clean [this] (clean this #{}))
   (clean [this what] (clean-puppetdb (service-context this) (get-config) what))
 
+  (delete-node [this certname]
+               (delete-node-from-puppetdb (service-context this) certname))
+
   (cmd-event-mult [this] (-> this service-context :cmd-event-mult)))
 
 (defn provide-services
